<?php

namespace Drupal\email_content_templates;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\user\EntityOwnerInterface;
use Drupal\Core\Entity\EntityChangedInterface;

/**
 * Provides an interface defining an email content template entity type.
 */
interface EmailContentTemplateInterface extends ContentEntityInterface, EntityOwnerInterface, EntityChangedInterface {

  /**
   * Gets the email content template title.
   *
   * @return string
   *   Title of the email content template.
   */
  public function getTitle();

  /**
   * Sets the email content template title.
   *
   * @param string $title
   *   The email content template title.
   *
   * @return \Drupal\email_content_templates\EmailContentTemplateInterface
   *   The called email content template entity.
   */
  public function setTitle($title);

  /**
   * Gets the email content template creation timestamp.
   *
   * @return int
   *   Creation timestamp of the email content template.
   */
  public function getCreatedTime();

  /**
   * Sets the email content template creation timestamp.
   *
   * @param int $timestamp
   *   The email content template creation timestamp.
   *
   * @return \Drupal\email_content_templates\EmailContentTemplateInterface
   *   The called email content template entity.
   */
  public function setCreatedTime($timestamp);

  /**
   * Returns the email content template status.
   *
   * @return bool
   *   TRUE if the email content template is enabled, FALSE otherwise.
   */
  public function isEnabled();

  /**
   * Sets the email content template status.
   *
   * @param bool $status
   *   TRUE to enable this email content template, FALSE to disable.
   *
   * @return \Drupal\email_content_templates\EmailContentTemplateInterface
   *   The called email content template entity.
   */
  public function setStatus($status);

}
