<?php

namespace Drupal\email_content_templates;

use Drupal\Core\Config\Entity\ConfigEntityListBuilder;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a class to build a listing of emailcontenttemplate type entities.
 *
 * @see \Drupal\email_content_templates\Entity\EmailContentTemplateType
 */
class EmailContentTemplateTypeListBuilder extends ConfigEntityListBuilder {


  /**
   * The email info manager.
   *
   * @var \Drupal\email_content_templates\EmailContentTemplatesInfoPluginManager
   */
  private $emailInfoManager;

  /**
   * Constructs a new EmailContentTemplateListBuilder object.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type definition.
   * @param \Drupal\Core\Entity\EntityStorageInterface $storage
   *   The entity storage class.
   * @param \Drupal\email_content_templates\EmailContentTemplatesInfoPluginManager $email_info_manager
   *   The email info manager.
   */
  public function __construct(EntityTypeInterface $entity_type, EntityStorageInterface $storage, EmailContentTemplatesInfoPluginManager $email_info_manager) {
    parent::__construct($entity_type, $storage);
    $this->emailInfoManager = $email_info_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    return new static(
      $entity_type,
      $container->get('entity_type.manager')->getStorage($entity_type->id()),
      $container->get('plugin.manager.email_content_templates_info')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header['title'] = $this->t('Label');
    $header['email_plugin'] = $this->t('Email plugin');

    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    $row['title'] = [
      'data' => $entity->label(),
      'class' => ['menu-label'],
    ];

    $plugin_label = '';
    $plugin_id = $entity->getEmailPlugin();
    if ($plugin_id !== '_none') {
      $plugin = $this->emailInfoManager->getDefinition($plugin_id);
    }

    if (!empty($plugin)) {
      $plugin_label = $plugin['label'];
    }

    $row['email_plugin'] = [
      'data' => $plugin_label,
    ];

    return $row + parent::buildRow($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function render() {
    $build = parent::render();

    $build['table']['#empty'] = $this->t(
      'No emailcontenttemplate types available. <a href=":link">Add emailcontenttemplate type</a>.',
      [
        ':link' => Url::fromRoute('entity.email_content_template_type.add_form')
          ->toString(),
      ]
    );

    return $build;
  }

}
