<?php

namespace Drupal\email_content_templates;

/**
 * Interface for email_content_templates_info plugins.
 */
interface EmailContentTemplatesInfoInterface {

  /**
   * Returns the translated plugin label.
   *
   * @return string
   *   The translated title.
   */
  public function label();

  /**
   * Returns the information so hook_mail parameters can be mapped to tokens.
   *
   * @return array
   *   The map.
   */
  public function paramTokenMap();

  /**
   * Returns the form used for testing the template.
   *
   * @return array
   *   The form array.
   */
  public function getTestForm(): array;

  /**
   * Checks if the given template should be used.
   *
   * @param \Drupal\email_content_templates\EmailContentTemplateInterface $template
   *   The template.
   * @param array $message
   *   The message.
   *
   * @return bool
   *   Whether the template should be used or not.
   */
  public function applies(EmailContentTemplateInterface $template, array $message): int;

  /**
   * Alters the mail after hook_mail() has run..
   *
   * @param \Drupal\email_content_templates\EmailContentTemplateInterface $template
   *   The template.
   * @param array $message
   *   The data from hook_mail().
   */
  public function alterMail(EmailContentTemplateInterface $template, array &$message): void;

}
