<?php

namespace Drupal\email_content_templates;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\token\TokenInterface;

/**
 * Provides Email Preparation functionality.
 */
class EmailContentTemplatesService implements EmailContentTemplatesServiceInterface {

  /**
   * Constructs a new EmailContentTemplatesService.
   */
  public function __construct(EmailContentTemplatesInfoPluginManager $email_info_manager, EntityTypeManagerInterface $entity_type_manager, RendererInterface $renderer, TokenInterface $token) {
    $this->emailInfoManager = $email_info_manager;
    $this->entityTypeManager = $entity_type_manager;
    $this->renderer = $renderer;
    $this->token = $token;
  }

  /**
   * {@inheritdoc}
   */
  public function prepareMessage(&$mail, $params, EmailContentTemplateInterface $template, EmailContentTemplatesInfoInterface $plugin = NULL) {
    $body = $template->get('body')->getValue();
    $body_text = $body[0]['value'];
    $subject = $template->get('subject')->getString();

    if (empty($plugin)) {
      $plugin = $this->emailInfoManager
        ->createInstance($template->getEmailPlugin());
    }

    if (!empty($plugin)) {
      $param_token_map = $plugin->paramTokenMap();
      $token_data = [];
      if (!empty($param_token_map)) {
        foreach ($param_token_map as $param => $token) {
          $token_data[$token] = $params[$param];
        }
      }
      $body_text = $this->token->replace($body_text, $token_data);
      $subject = $this->token->replace($subject, $token_data);
    }

    /** @var \Drupal\filter\FilterFormatInterface $format */
    $format = $this->entityTypeManager
      ->getStorage('filter_format')
      ->load($body[0]['format']);
    $filters = $format->filters();
    $html_mail = TRUE;
    if ($filters->has('filter_html_escape')) {
      $filter = $filters->get('filter_html_escape');
      $html_mail = !$filter->status;
    }

    if ($html_mail) {
      $render = $this->entityTypeManager
        ->getViewBuilder($template->getEntityTypeId())
        ->view($template);
      $body = $this->renderer->renderPlain($render);
      $mail['headers']['Content-Type'] = 'text/html';
    }
    else {
      $render = [
        '#type' => 'processed_text',
        '#text' => $body_text,
        '#format' => $body[0]['format'],
      ];
      $body = $this->renderer->renderPlain($render);
    }
    $mail['body'][0] = $body;
    $mail['subject'] = $subject;
    $plugin->alterMail($template, $mail);
  }

}
