<?php

namespace Drupal\email_content_templates\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBundleBase;
use Drupal\Core\Entity\EntityDescriptionInterface;

/**
 * Defines the EmailContentTemplate type configuration entity.
 *
 * @ConfigEntityType(
 *   id = "email_content_template_type",
 *   label = @Translation("EmailContentTemplate type"),
 *   handlers = {
 *     "form" = {
 *       "add" =
 *   "Drupal\email_content_templates\Form\EmailContentTemplateTypeForm",
 *       "edit" =
 *   "Drupal\email_content_templates\Form\EmailContentTemplateTypeForm",
 *       "delete" = "Drupal\Core\Entity\EntityDeleteForm",
 *     },
 *     "list_builder" =
 *   "Drupal\email_content_templates\EmailContentTemplateTypeListBuilder",
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *     }
 *   },
 *   admin_permission = "administer emailcontenttemplate types",
 *   bundle_of = "email_content_template",
 *   config_prefix = "email_content_template_type",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *     "uuid" = "uuid"
 *   },
 *   links = {
 *     "add-form" = "/admin/structure/email_content_template_types/add",
 *     "edit-form" =
 *   "/admin/structure/email_content_template_types/manage/{email_content_template_type}",
 *     "delete-form" =
 *   "/admin/structure/email_content_template_types/manage/{email_content_template_type}/delete",
 *     "collection" = "/admin/structure/email_content_template_types"
 *   },
 *   config_export = {
 *     "id",
 *     "label",
 *     "uuid",
 *     "description",
 *     "default_subject",
 *     "default_body",
 *     "default_body_format",
 *     "email_plugin"
 *   }
 * )
 */
class EmailContentTemplateType extends ConfigEntityBundleBase implements EntityDescriptionInterface {

  /**
   * The machine name of this emailcontenttemplate type.
   *
   * @var string
   */
  protected $id;

  /**
   * The human-readable name of the type.
   *
   * @var string
   */
  protected $label;

  /**
   * The id of the used email plugin.
   *
   * @var string
   */
  protected $email_plugin;

  /**
   * The bundle description.
   *
   * @var string
   */
  protected $description;

  /**
   * The default title.
   *
   * @var string
   */
  protected $default_subject;

  /**
   * The default body.
   *
   * @var string
   */
  protected $default_body;

  /**
   * The default body format.
   *
   * @var string
   */
  protected $default_subject_format;

  /**
   * Sets the email plugin.
   *
   * @param string $email_plugin
   *   The email plugin.
   */
  public function setEmailPlugin($email_plugin) {
    $this->email_plugin = $email_plugin;
  }

  /**
   * Returns the email plugin.
   *
   * @return string
   *   The email plugin.
   */
  public function getEmailPlugin() {
    return $this->email_plugin;
  }

  /**
   * {@inheritdoc}
   */
  public function setDescription($description) {
    $this->description = $description;
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    return $this->description;
  }

  /**
   * Sets the default template title.
   *
   * @param string $default_subject
   *   The default title.
   */
  public function setDefaultSubject($default_subject) {
    $this->default_subject = $default_subject;
  }

  /**
   * Returns the default template title.
   *
   * @return string
   *   The default template title.
   */
  public function getDefaultSubject() {
    return $this->default_subject;
  }

  /**
   * Sets the default template body.
   *
   * @param string $default_body
   *   The default body.
   */
  public function setDefaultBody($default_body) {
    $this->default_body = $default_body;
  }

  /**
   * Returns the default template body.
   *
   * @return string
   *   The default template body.
   */
  public function getDefaultBody() {
    return $this->default_body;
  }

  /**
   * Sets the default template body format.
   *
   * @param string $default_body_format
   *   The default body format.
   */
  public function setDefaultBodyFormat($default_body_format) {
    $this->default_body_format = $default_body_format;
  }

  /**
   * Returns the default template body.
   *
   * @return string
   *   The default template body format.
   */
  public function getDefaultBodyFormat() {
    return $this->default_body_format;
  }

}
